@@ -46,8 +46,7 @@
 ##'   (default TRUE) turn of additional output, \code{show.plot}
 ##'   (default FALSE) show diagnostic plots.
 ##' 
-##' @return returns a vector with the same length as models containing
-##'   the stability scores
+##' @return returns a list with the entries "scores" and "parameter"
 ##'
 ##' @export
 ##' 
@@ -78,10 +77,10 @@
 ##'
 ##' ## Fit data to the following two models using CausalKinetiX:
 ##' ## 1: dy = theta_1*x_1 + theta_2*x_2 + theta_3*x_1*x_10 (true model)
-##' ## 2: dy = theta_1*x_2 + theta_2*x_3*x_10 (wrong model)
+##' ## 2: dy = theta_1*x_2 + theta_2*x_4 + theta_3*x_3*x_10 (wrong model)
 ##' ck.fit <- CausalKinetiX.modelranking(D, time, env, target,
 ##'                                      list(list(1, 2, c(1, 10)), list(2, 4, c(3, 10))))
-##' print(ck.fit)
+##' print(ck.fit$scores)
 
 
 
@@ -152,17 +151,29 @@ CausalKinetiX.modelranking <- function(D,
   if(pars$smooth.Y & !is.na(pars$splitting.env)){
     stop("If smooth.Y is TRUE, splitting.env needs to be NA.")
   }
-
   
   ############################
   #
   # initialize
   #
   ############################
+
+  # remove all variables not involved in any models
+  unique_vars <- unique(c(unlist(lapply(models, unlist)), target, pars$include.vars))
+  unique_vars <- sort(unique_vars[!is.na(unique_vars)])
+  L <- length(times)
+  matching_perm <- order(c(unique_vars, setdiff(1:(ncol(D)/L), unique_vars)))
+  target <- matching_perm[target]
+  models <- lapply(models, function(x) lapply(x, function(y) matching_perm[y]))
+  keep_ind <- do.call(c, lapply(unique_vars, function(i) ((i-1)*L+1):(i*L)))
+  D <- D[,keep_ind]
+  if(!is.na(pars$include.vars)[1]){
+    pars$include.vars <- matching_perm[pars$include.vars]
+  }
   
   # read out parameters (as in paper)
-  n <- nrow(D)
   L <- length(times)
+  n <- nrow(D)
   d <- ncol(D)/L
   num.folds <- pars$num.folds
   pen.degree <- pars$pen.degree
@@ -183,9 +194,10 @@ CausalKinetiX.modelranking <- function(D,
   show.plot <- pars$show.plot
 
   # check whether to include products and interactions
-  products <- sum(sapply(models, function(model)
+  non_zero_mod <- sapply(models, function(x) length(x)>0)
+  products <- sum(sapply(models[non_zero_mod], function(model)
     sum(sapply(model, function(term) length(term) > length(unique(term)))) > 0)) > 0
-  interactions <- sum(sapply(models, function(model)
+  interactions <- sum(sapply(models[non_zero_mod], function(model)
     sum(sapply(model, function(term) length(unique(term)) > 1)) > 0)) > 0
 
   # sort environments to increasing order
@@ -225,7 +237,6 @@ CausalKinetiX.modelranking <- function(D,
     dtot <- d
     ordering <- as.list(1:d)
   }
-
   
   ##################################################
   #
@@ -475,6 +486,7 @@ CausalKinetiX.modelranking <- function(D,
   
   # Iterate over all models and compute score
   scores <- vector("numeric", num.models)
+  parameters <- vector("list", num.models)
   for(model in 1:num.models){
     Xlist <- data_list[[model]]
     Xlist2 <- data_list2[[model]]
@@ -518,11 +530,12 @@ CausalKinetiX.modelranking <- function(D,
         coefs <- coefficients(fit)
         coefs[is.na(coefs)] <- 0
         fitted_dY <- Xpred %*% matrix(coefs, ncol(Xpred), 1)
+        parameters[[model]] <- coefs
       }
       # OLS regression with sign constraints on parameters
       else if(regression.class=="signed.OLS"){
-        len_model <- length(modelstot[model][[1]])
-        tmp <- sapply(modelstot[model][[1]], function(x) x[1])
+        len_model <- length(modelstot[[model]])
+        tmp <- sapply(modelstot[[model]], function(x) x[1])
         ind <- rep(1, len_model)
         ind[tmp==target] <- -1
         # define quadProg parameters
@@ -530,9 +543,11 @@ CausalKinetiX.modelranking <- function(D,
         Amat <- diag(ind)
         dvec <- matrix(dY, 1, nrow(X)) %*% X
         Dmat <- (t(X) %*% X)
-        fit <- solve.QP(Dmat, dvec, Amat, bvec, meq=0)
+        sc <- norm(Dmat,"2")
+        fit <- solve.QP(Dmat/sc, dvec/sc, Amat, bvec, meq=0)
         coefs <- fit$solution
         fitted_dY <- Xpred %*% matrix(coefs, ncol(Xpred), 1)
+        parameters[[model]] <- coefs
       }
       # OLS regression with pruning based on score
       else if(regression.class=="optim"){
@@ -543,15 +558,16 @@ CausalKinetiX.modelranking <- function(D,
           return(max(sapply(split(tmp_vec, env_vec), mean)))
         }
         # compute starting value using quadratic program
-        len_model <- length(modelstot[model][[1]])
-        tmp <- sapply(modelstot[model][[1]], function(x) x[1])
+        len_model <- length(modelstot[[model]])
+        tmp <- sapply(modelstot[[model]], function(x) x[1])
         ind <- rep(1, len_model)
         ind[tmp==target] <- -1
         bvec <- rep(10^(-10), len_model)
         Amat <- diag(ind)
         dvec <- matrix(dY, 1, nrow(X)) %*% X
         Dmat <- (t(X) %*% X)
-        fit <- solve.QP(Dmat, dvec, Amat, bvec, meq=0)
+        sc <- norm(Dmat,"2")
+        fit <- solve.QP(Dmat/sc, dvec/sc, Amat, bvec, meq=0)
         coefs <- fit$solution
         # perform optimization
         opt.res <- optim(log(coefs*ind)/log(10), loss_fun,
@@ -562,15 +578,17 @@ CausalKinetiX.modelranking <- function(D,
                          lower=-10, upper=5)
         coefs <- (10^opt.res$par)*ind
         fitted_dY <- Xpred %*% matrix(coefs, ncol(Xpred), 1)
+        parameters[[model]] <- coefs
       }
       # Random forest regression
       else if(regression.class=="random.forest"){
         fit <- randomForest(y ~ ., data=cbind(as.data.frame(X), data.frame(y=dY)))
         fitted_dY <- predict(fit, newdata=as.data.frame(Xpred))
+        parameters[[model]] <- fit
       }
       # Wrong regression.class
       else{
-        stop("Specified regression.class does not exist. Use OLS, OLS.prune or random.forest.")
+        stop("Specified regression.class does not exist. Use OLS, signed.OLS, optim or random.forest.")
       }
       # compute score using splitting environment
       if(!smooth.Y){
@@ -694,6 +712,7 @@ CausalKinetiX.modelranking <- function(D,
         maxyplot <- vector("numeric", length(RSS_A))
         constrained_fit <- vector("list", length(RSS_A))
       }
+      parameters[[model]] <- vector("list", num.env)
       for(i in 1:num.env){
         # compute derivative constraint using OLS
         dYout <- dY[loo.ind!=unique_env[i]]
@@ -715,21 +734,29 @@ CausalKinetiX.modelranking <- function(D,
           coefs <- coefficients(fit)
           coefs[is.na(coefs)] <- 0
           fitted_dY <- Xin %*% matrix(coefs, ncol(Xin), 1)
+          parameters[[model]][[i]] <- coefs
         }
         # OLS regression with sign constraints on parameters
         else if(regression.class=="signed.OLS"){
-          len_model <- length(modelstot[model][[1]])
-          tmp <- sapply(modelstot[model][[1]], function(x) x[1])
+          len_model <- length(modelstot[[model]])
+          tmp <- sapply(modelstot[[model]], function(x) x[1])
           ind <- rep(1, len_model)
           ind[tmp==target] <- -1
           # define quadProg parameters
           bvec <- rep(0, len_model)
-          Amat <- diag(ind)
+          if(length(ind)==1){
+            Amat <- as.matrix(ind)
+          }
+          else{
+            Amat <- diag(ind)
+          }
           dvec <- matrix(dYout_nona, 1, nrow(Xout_nona)) %*% Xout_nona
           Dmat <- (t(Xout_nona) %*% Xout_nona)
-          fit <- solve.QP(Dmat, dvec, Amat, bvec, meq=0)
+          sc <- norm(Dmat,"2")
+          fit <- solve.QP(Dmat/sc, dvec/sc, Amat, bvec, meq=0)
           coefs <- fit$solution
           fitted_dY <- Xin %*% matrix(coefs, ncol(Xin), 1)
+          parameters[[model]][[i]] <- coefs
         }
         # OLS regression with pruning based on score
         else if(regression.class=="optim"){
@@ -740,15 +767,16 @@ CausalKinetiX.modelranking <- function(D,
             return(mean(sapply(split(tmp_vec, env_vec), mean)))
           }
           # compute starting value using quadratic program
-          len_model <- length(modelstot[model][[1]])
-          tmp <- sapply(modelstot[model][[1]], function(x) x[1])
+          len_model <- length(modelstot[[model]])
+          tmp <- sapply(modelstot[[model]], function(x) x[1])
           ind <- rep(1, len_model)
           ind[tmp==target] <- -1
           bvec <- rep(10^(-10), len_model)
           Amat <- diag(ind)
           dvec <- matrix(dYout_nona, 1, nrow(Xout_nona)) %*% Xout_nona
           Dmat <- (t(Xout_nona) %*% Xout_nona)
-          fit <- solve.QP(Dmat, dvec, Amat, bvec, meq=0)
+          sc <- norm(Dmat,"2")
+          fit <- solve.QP(Dmat/sc, dvec/sc, Amat, bvec, meq=0)
           coefs <- fit$solution
           # perform optimization
           opt.res <- optim(log(coefs*ind)/log(10), loss_fun,
@@ -759,15 +787,17 @@ CausalKinetiX.modelranking <- function(D,
                            lower=-10, upper=5)
           coefs <- (10^opt.res$par)*ind
           fitted_dY <- Xin %*% matrix(coefs, ncol(Xin), 1)
+          parameters[[model]][[i]] <- coefs
         }
         # Random forest regression
         else if(regression.class=="random.forest"){
           fit <- randomForest(y ~ ., data=cbind(as.data.frame(Xout_nona), data.frame(y=dYout_nona)))
           fitted_dY <- predict(fit, newdata=as.data.frame(Xin))
+          parameters[[model]][[i]] <- fit
         }
         # Wrong regression.class
         else{
-          stop("Specified regression.class does not exist. Use OLS, OLS.prune or random.forest.")
+          stop("Specified regression.class does not exist. Use OLS, signed.OLS, optim or random.forest.")
         }
         
         # Fit individual models with derivative constraint
@@ -863,13 +893,13 @@ CausalKinetiX.modelranking <- function(D,
     }
     ## compute score
     if(score.type=="max"){
-      if(abs(RSS_A)<10^-10){
+      if(min(abs(RSS_A))<10^-10){
         warning("RSS of unconstrained smoother is very small (<10^-10). Using a relative score will lead to wrong results. Consider using the score.type max_absolut.")
       }
       score <- max((RSS_B-RSS_A)/RSS_A)
     }
     else if(score.type=="mean"){
-      if(abs(RSS_A)<10^-10){
+      if(min(abs(RSS_A))<10^-10){
         warning("RSS of unconstrained smoother is very small (<10^-10). Using a relative score will lead to wrong results. Consider using the score.type mean_absolut.")
       }
       score <- mean((RSS_B-RSS_A)/RSS_A)
@@ -881,11 +911,15 @@ CausalKinetiX.modelranking <- function(D,
       score <- max(RSS_B)
     }
     else if(score.type=="mean.weighted"){
-      if(abs(RSS_A)<10^-10){
+      if(min(abs(RSS_A))<10^-10){
         warning("RSS of unconstrained smoother is very small (<10^-10). Using a relative score will lead to wrong results. Consider using the score.types mean_absolut or max_absolute.")
       }
       score <- mean(weight.vec*(RSS_B-RSS_A)/RSS_A)
     }
+    else if(score.type=="stability"){
+      pairs <- combn(1:length(RSS_B), 2)
+      score <- max(apply(pairs, 2, function(x) abs(RSS_B[x[1]]-RSS_B[x[2]])))
+    }
     else{
       stop("Specified score.type does not exist. Use max, mean or max-mean.")
     }
@@ -897,6 +931,7 @@ CausalKinetiX.modelranking <- function(D,
   }
 
   ## Return results
-  return(scores)
+  return(list(scores=scores,
+              parameters=parameters))
 
 }
